'use strict';

var fs = require('fs');
var upath = require('upath');
var lodash = require('lodash');
var tag = require('./tag-5dd23b65.js');

//import { loader } from 'webpack'
function parseSfc$1(compiler) {
  var filename = upath.basename(this.resourcePath);
  var context = this.rootContext || process.cwd();
  var sourceRoot = upath.dirname(upath.relative(context, this.resourcePath));
  return compiler.parse(fs.readFileSync(sourceRoot + '/' + filename, 'utf8')).descriptor;
}

function compileTemplateFromDescriptor$1(sfcDescriptor, compiler) {
  var _sfcDescriptor$templa;

  // check we have a template to work with
  if (!(sfcDescriptor !== null && sfcDescriptor !== void 0 && (_sfcDescriptor$templa = sfcDescriptor.template) !== null && _sfcDescriptor$templa !== void 0 && _sfcDescriptor$templa.content)) {
    return;
  }

  var content = sfcDescriptor.template.content; // need to compile pug to html so that we can compile it using the vue/compiler-sfc

  if (sfcDescriptor.template.lang === 'pug') {
    var pug = require('pug');

    content = pug.render(content, {
      filename: this.resourcePath
    });
  }

  var shortFilePath = this.resourcePath.replace(/^(\.\.[\/\\])+/, '').replace(/\\/g, '/');
  return compiler.compileTemplate({
    id: shortFilePath,
    source: content,
    filename: this.resourcePath,
    prettify: false
  });
}

//import { loader } from 'webpack'
//import { SFCTemplateCompileResults } from '@vue/compiler-sfc'

function extractTagsFromSfc$1(options) {
  // parse the SFC component and get a descriptor
  var sfcDescriptor = parseSfc$1.call(this, options.compiler); // compile the template content from the descriptor

  var compiled = compileTemplateFromDescriptor$1.call(this, sfcDescriptor, options.compiler);

  if (compiled && compiled.ast) {
    var tags = compiled.ast.components; // return all unique tags as kebab case

    return lodash.uniq(lodash.map(tags, function (tag) {
      return lodash.kebabCase(tag);
    }));
  }

  return;
}

function parseSfc(compiler) {
  var filename = upath.basename(this.resourcePath);
  var context = this.rootContext || process.cwd();
  var sourceRoot = upath.dirname(upath.relative(context, this.resourcePath));
  return compiler.parseComponent(fs.readFileSync(sourceRoot + '/' + filename, 'utf8'));
}

//import { loader } from 'webpack'
function compileTemplateFromDescriptor(sfcDescriptor, compiler) {
  var _sfcDescriptor$templa;

  // check we have a template to work with
  if (!(sfcDescriptor !== null && sfcDescriptor !== void 0 && (_sfcDescriptor$templa = sfcDescriptor.template) !== null && _sfcDescriptor$templa !== void 0 && _sfcDescriptor$templa.content)) {
    return;
  }

  var content = sfcDescriptor.template.content; // need to compile pug to html so that we can compile it using the vue/compiler-sfc

  if (sfcDescriptor.template.lang === 'pug') {
    var pug = require('pug');

    content = pug.render(content, {
      filename: this.resourcePath
    });
  }

  var tags = [];
  var compiled = compiler.compile(content, {
    modules: [{
      postTransformNode: function postTransformNode(node) {
        tags.push(node.tag);
      }
    }]
  });
  compiled.tags = tags;
  return compiled;
}

function extractTagsFromSfc(options) {
  // parse the SFC component and get a descriptor
  var sfcDescriptor = parseSfc.call(this, options.compiler); // compile the template content from the descriptor

  var compiled = compileTemplateFromDescriptor.call(this, sfcDescriptor, options.compiler);

  if (compiled && compiled.tags) {
    // return all unique tags as kebab case
    return lodash.uniq(lodash.map(compiled.tags, function (tag) {
      return lodash.kebabCase(tag);
    }));
  }

  return;
}

var _require = require('@vue/cli-shared-utils'),
    semver = _require.semver,
    loadModule = _require.loadModule,
    error = _require.error,
    info = _require.info;

function loadVue2TemplateCompiler(api) {
  return loadModule('vue-template-compiler', api.service.context) || loadModule('vue-template-compiler', __dirname);
}

function loadVue3Compiler(api) {
  return loadModule('@vue/compiler-sfc', api.service.context) || loadModule('@vue/compiler-sfc', __dirname);
}

var plugin = function plugin(api, options) {
  var _options$pluginOption;

  var pluginOptions = Object.assign( //default configuration
  {
    path: './src/components',
    extensions: ['vue', 'js', 'ts'],
    ignore: ['**/*.stories.js' // ignore storybook files
    ],
    rules: [] //{ pattern: '^a-(.*)', importer: 'import $1 from "ant-design-vue/es/$2"', installer: 'Vue.use($1)' }
    //$0 it is the tag's kebab name
    //$1 it is the tag's pascal name
    //$2 it is the tag matched part kebab name.
    //$3 it is the tag matched part pascal name.

  }, // users provided configuration
  (_options$pluginOption = options.pluginOptions) === null || _options$pluginOption === void 0 ? void 0 : _options$pluginOption.easyimport);

  if (!pluginOptions.pattern) {
    pluginOptions.pattern = "**/*.{".concat(pluginOptions.extensions.join(','), ",}");
  } // resolve the configured path


  pluginOptions.path = api.resolve(pluginOptions.path); // check the components path exists
  // if (!existsSync(pluginOptions.path)) {
  //   // if not we should create it so we don't error out
  //   mkdirSync(pluginOptions.path)
  //   // warn the user, possible misconfiguration?
  //   info('The components path "' + pluginOptions.path + '" was created.', '')
  // }

  if (pluginOptions.desc) {
    info(pluginOptions.desc, tag.TAG);
  }

  info(JSON.stringify(pluginOptions), tag.TAG);
  var vue = loadModule('vue', api.service.context) || loadModule('vue', __dirname);

  if (!vue) {
    error('Aborting, failed to load vue module.', tag.TAG);
    return;
  }

  var vueVersion = semver.major(vue.version);
  pluginOptions.vueVersion = vueVersion;

  if (vueVersion === 2) {
    pluginOptions.extractor = extractTagsFromSfc;
    pluginOptions.compiler = loadVue2TemplateCompiler(api);
  } else if (vueVersion >= 3) {
    pluginOptions.extractor = extractTagsFromSfc$1;
    pluginOptions.compiler = loadVue3Compiler(api);
  } else {
    error('Aborting, vue version ' + vueVersion + ' not supported', tag.TAG);
    return;
  } // extend the base webpack configuration


  api.chainWebpack(function (webpackConfig) {
    // use oneOf so we're not running the loader when it's not required?
    webpackConfig.module.rules.get('vue') // add our custom components loader before the vue-loader
    .use('easyimport').loader(require.resolve('./loader')).options(pluginOptions).before('vue-loader').end();
  }); // need to return something for typescript

  return true;
};

module.exports = plugin;
