'use strict';

var loaderUtils = require('loader-utils');
var upath = require('upath');
var globby = require('globby');
var lodash = require('lodash');
var fs = require('fs');
var tag = require('./tag-5dd23b65.js');
var cliSharedUtils = require('@vue/cli-shared-utils');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var globby__default = /*#__PURE__*/_interopDefaultLegacy(globby);

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelper(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];

  if (!it) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;

      var F = function () {};

      return {
        s: F,
        n: function () {
          if (i >= o.length) return {
            done: true
          };
          return {
            done: false,
            value: o[i++]
          };
        },
        e: function (e) {
          throw e;
        },
        f: F
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  var normalCompletion = true,
      didErr = false,
      err;
  return {
    s: function () {
      it = it.call(o);
    },
    n: function () {
      var step = it.next();
      normalCompletion = step.done;
      return step;
    },
    e: function (e) {
      didErr = true;
      err = e;
    },
    f: function () {
      try {
        if (!normalCompletion && it.return != null) it.return();
      } finally {
        if (didErr) throw err;
      }
    }
  };
}

function _await$1(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

var scanComponents = _async$1(function (options, srcDir) {
  var path, pattern, _options$ignore, ignore, _iterator, _step, _loop, _ret;

  var components = [];
  path = options.path;
  pattern = options.pattern;
  _options$ignore = options.ignore;
  ignore = _options$ignore === void 0 ? [] : _options$ignore;
  if (!fs.existsSync(path)) return components;
  var filePaths = new Set();
  var scannedPaths = [];
  var resolvedNames = new Map();
  var _createForOfIteratorH = _createForOfIteratorHelper;
  return _await$1(globby__default['default'](pattern, {
    cwd: path,
    ignore: ignore
  }), function (_globby) {
    _iterator = _createForOfIteratorH(_globby);

    try {
      _loop = function _loop() {
        var _file = _step.value;
        var filePath = upath.resolve(upath.join(path, _file));

        if (scannedPaths.find(function (d) {
          return filePath.startsWith(d);
        })) {
          return "continue";
        }

        if (filePaths.has(filePath)) {
          return "continue";
        }

        filePaths.add(filePath);
        var fileName = upath.basename(filePath, upath.extname(filePath));

        if (fileName === 'index') {
          fileName = upath.basename(upath.dirname(filePath), upath.extname(filePath));
        }

        if (resolvedNames.has(fileName)) {
          // eslint-disable-next-line no-console
          console.warn("Two component files resolving to the same name `".concat(fileName, "`:\n") + "\n - ".concat(filePath) + "\n - ".concat(resolvedNames.get(fileName)));
          return "continue";
        }

        resolvedNames.set(fileName, filePath);
        var pascalName = pascalCase$1(fileName);
        var kebabName = lodash.kebabCase(fileName);
        var shortPath = filePath.replace(srcDir, '@');
        var component = {
          filePath: filePath,
          pascalName: pascalName,
          kebabName: kebabName,
          shortPath: shortPath
        };
        var resolvedComponent = component; // allow someone to configure the component mapping

        if (options.mapComponent) {
          resolvedComponent = options.mapComponent(component);
        } // if a component was resolved as false, then we shouldn't push it


        if (resolvedComponent) {
          // resolve the import variable in case the user wanted to modify paths or the casing
          resolvedComponent["import"] = "import ".concat(pascalName, " from \"").concat(shortPath, "\";");
          components.push(resolvedComponent);
        }
      };

      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        _ret = _loop();
        if (_ret === "continue") return;
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }

    scannedPaths.push(path);
    return components;
  });
});

function _async$1(f) {
  return function () {
    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

var pascalCase$1 = function pascalCase(str) {
  return lodash.upperFirst(lodash.camelCase(str));
};

var pascalCase = function pascalCase(str) {
  return lodash.upperFirst(lodash.camelCase(str));
};

function replace(str, searcher, replacement) {
  return str.split(searcher).join(replacement);
}

function genComponet(tagKebabName, rule, regExp) {
  var matchedKebabName = lodash.kebabCase(regExp[1]);
  var matchedPascalName = pascalCase(matchedKebabName);
  var tagPascalName = pascalCase(tagKebabName);
  var importer = genComponetImport(tagKebabName, tagPascalName, matchedKebabName, matchedPascalName, rule);
  var installer = genComponetInstaller(tagKebabName, tagPascalName, matchedKebabName, matchedPascalName, rule);
  if (!importer) return false;
  return {
    "import": importer,
    install: installer
  };
}

function genComponetInstaller(tagKebabName, tagPascalName, matchedKebabName, matchedPascalName, rule) {
  var str;

  if (lodash.isFunction(rule.installer)) {
    str = rule.installer(tagKebabName, tagPascalName, matchedKebabName, matchedPascalName);
  } else {
    str = replace(rule.installer, '$0', tagKebabName);
    str = replace(str, '$1', tagPascalName);
    str = replace(str, '$2', matchedKebabName);
    str = replace(str, '$3', matchedPascalName);
  }

  if (str && str.charAt(str.length - 1) != ';') {
    str = str + ';';
  }

  return str;
}

function genComponetImport(tagKebabName, tagPascalName, matchedKebabName, matchedPascalName, rule) {
  var str;

  if (lodash.isFunction(rule.importer)) {
    str = rule.importer(tagKebabName, tagPascalName, matchedKebabName, matchedPascalName);
  } else {
    str = replace(rule.importer, '$0', tagKebabName);
    str = replace(str, '$1', tagPascalName);
    str = replace(str, '$2', matchedKebabName);
    str = replace(str, '$3', matchedPascalName);
  }

  if (!str) return false;

  if (!lodash.startsWith(str, 'import ')) {
    str = 'import ' + str;
  }

  if (str.charAt(str.length - 1) != ';') {
    str = str + ';';
  }

  return str;
}

function ruledMatcher (kebabTag, options) {
  if (!(Array.isArray(options.rules) && options.rules.length > 0)) {
    return;
  }

  var component;
  options.rules.forEach(function (rule) {
    if (rule.pattern && rule.importer) {
      var p = new RegExp(rule.pattern);
      var matched = p.exec(kebabTag);

      if (matched) {
        component = genComponet(kebabTag, rule, matched);
      }
    }
  });
  return component;
}

/**
 * Match tags to components.
 *
 * @param tags An array of unique tags, should be unique for casing already
 * @param componentsToMatch Array of components from your project
 */

function matcher(tags, componentsToMatch, options) {
  var components = [];
  tags.forEach(function (tag) {
    var component = lodash.first(lodash.filter(componentsToMatch, function (component) {
      return tag === component.pascalName || tag === component.kebabName;
    }));

    if (component) {
      components.push(component);
    } else {
      var ruledComponent = ruledMatcher(tag, options);

      if (ruledComponent) {
        components.push(ruledComponent);
      }
    }
  });
  return components;
}

//import { info } from '@vue/cli-shared-utils'
function injectComponents(source, components) {
  var importers = [],
      installers = [],
      coms = [];
  components.forEach(function (item) {
    if (item["import"]) {
      importers.push(item["import"]);
    }

    if (item.pascalName) {
      coms.push(item.pascalName);
    }

    if (item.install) {
      installers.push(item.install);
    }
  });

  if (installers.length > 0) {
    importers.push('import Vue from "vue";');
  }

  importers = importers.concat(installers);
  var newContent = "/* ".concat(tag.TAG, " Begin */\n    ").concat(importers.join('\n'), "\n    script.components = Object.assign({}, { ").concat(coms.join(', '), " }, script.components);\n    /* ").concat(tag.TAG, " End */");
  var hotReload = source.indexOf('/* hot reload */');

  if (hotReload > -1) {
    source = source.slice(0, hotReload) + newContent + '\n\n' + source.slice(hotReload);
  } else {
    source += '\n\n' + newContent;
  }

  return source;
}

function _await(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

function _async(f) {
  return function () {
    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

var loader = _async(function (source) {
  var _this = this;

  //info(TAG + ' loader is runing', TAG)
  _this.cacheable();

  var loaderContext = _this; // bail out if we're dealing with a resource with a query
  // we only want to be dealing with the 'virtual-module' of the SFC

  if (loaderContext.resourceQuery) {
    return source;
  }

  var options = loaderUtils.getOptions(loaderContext);
  var tags = options.extractor.call(_this, options); // we only need to match the tags if we have some

  return !tags || tags.length <= 0 ? source : _await(scanComponents(options, upath.resolve('./src')), function (scannedComponents) {
    // make sure cache invalidation and recompile in watch mode
    scannedComponents.forEach(function (c) {
      return _this.addDependency(c.filePath);
    }); // the components to import

    var components = matcher(tags, scannedComponents, options); // only if we have components to inject

    if (components.length <= 0) {
      return source;
    }

    var injectedSource = injectComponents(source, components);

    if (options.debug) {
      cliSharedUtils.info('\r\n' + injectedSource, tag.TAG);
    }

    return injectedSource;
  });
});

module.exports = loader;
